#version 430
#extension GL_ARB_separate_shader_objects : enable










struct SComputeInput
{
	uvec3 GlobalThreadID; uvec3 LocalThreadID; uvec3 GroupID;};

layout(std430 ) readonly buffer BInIndirection
{
	uint InIndirection[];
} InIndirection;
layout(std430 ) readonly buffer BInKeys
{
	uint InKeys[];
} InKeys;
layout(std430 ) buffer BOutCounts
{
	uint OutCounts[];
} OutCounts;

layout( std140) uniform UInfo
{
	uint CurrentBit;
} Info;

shared uint localCounts[256];

void ComputeMain( in SComputeInput  cInput  )
{
	const uint globalThreadID = cInput.GlobalThreadID.x;
	const uint localThreadID = cInput.LocalThreadID.x;
	const uint groupID = cInput.GroupID.x;

	const uint currentBit =  Info.CurrentBit ;


	const uvec4 fullKeys64 =  uvec4( InKeys.InKeys [ globalThreadID * 4 ], InKeys.InKeys [( globalThreadID * 4 ) + 1], InKeys.InKeys [( globalThreadID * 4 ) + 2], InKeys.InKeys ( globalThreadID * 4 ) + 3]) ;
	uvec2 keyBits;
	keyBits.x = ((currentBit < 32 ? fullKeys64.y : fullKeys64.x) >> (currentBit % 32)) & 1;
	keyBits.y = ((currentBit < 32 ? fullKeys64.w : fullKeys64.z) >> (currentBit % 32)) & 1;
	const uvec2 startValues = 1 - keyBits;
	const uvec2 k01 = {0, 1};
	{
		uvec2 i = localThreadID * 2 + k01;
		localCounts [i.x] = startValues.x;
		localCounts [i.y] = startValues.y;
	}
	uint stride = 1;
	for (uint d =  128 ; d > 0; d /= 2)
	{
		groupMemoryBarrier(); barrier(); ;
		if (localThreadID < d)
		{
			uint ai = (localThreadID * 2 + 1) * stride - 1;
			uint bi = (localThreadID * 2 + 2) * stride - 1;
			localCounts [bi] +=  localCounts [ai];
		}
		stride *= 2;
	}
	groupMemoryBarrier(); barrier(); ;

	if (localThreadID == 0)
	{
		const uint total =  localCounts [ 128  * 2 - 1];
		OutCounts.OutCounts [ groupID ] = total; ;
	}
}

layout(local_size_x = 128, local_size_y = 1, local_size_z = 1) in;
void main()
{
	SComputeInput cInput;
	cInput.GlobalThreadID = gl_GlobalInvocationID;
	cInput.LocalThreadID = gl_LocalInvocationID;
	cInput.GroupID = gl_WorkGroupID;
	ComputeMain(cInput);
}
